@@ -1,5 +1,6 @@
 //! Configuring the shape of generated Wasm modules.
 
+use crate::InstructionKinds;
 use arbitrary::{Arbitrary, Result, Unstructured};
 
 /// Configuration for a generated module.
@@ -48,6 +49,16 @@ pub trait Config: 'static + std::fmt::Debug {
         100
     }
 
+    /// The minimum number of tags to generate. Defaults to 0.
+    fn min_tags(&self) -> usize {
+        0
+    }
+
+    /// The maximum number of tags to generate. Defaults to 100.
+    fn max_tags(&self) -> usize {
+        100
+    }
+
     /// The minimum number of functions to generate. Defaults to 0.  This
     /// includes imported functions.
     fn min_funcs(&self) -> usize {
@@ -238,11 +249,23 @@ pub trait Config: 'static + std::fmt::Debug {
     }
 
     /// Determines whether the SIMD proposal is enabled for
-    /// generating insructions. Defaults to `true`.
+    /// generating insructions. Defaults to `false`.
     fn simd_enabled(&self) -> bool {
         false
     }
 
+    /// Determines whether the Relaxed SIMD proposal is enabled for
+    /// generating insructions. Defaults to `false`.
+    fn relaxed_simd_enabled(&self) -> bool {
+        false
+    }
+
+    /// Determines whether the exception-handling proposal is enabled for
+    /// generating insructions. Defaults to `false`.
+    fn exceptions_enabled(&self) -> bool {
+        false
+    }
+
     /// Determines whether the module linking proposal is enabled.
     ///
     /// Defaults to `false`.
@@ -297,6 +320,21 @@ pub trait Config: 'static + std::fmt::Debug {
     fn canonicalize_nans(&self) -> bool {
         false
     }
+
+    /// Returns the kinds of instructions allowed in the generated wasm
+    /// programs.
+    ///
+    /// The categories of instructions match the categories used by the
+    /// [WebAssembly
+    /// specification](https://webassembly.github.io/spec/core/syntax/instructions.html);
+    /// e.g., numeric, vector, control, memory, etc. Note that modifying this
+    /// setting is separate from the proposal flags; that is, if `simd_enabled()
+    /// == true` but `allowed_instruction()` does not include vector
+    /// instructions, the generated programs will not include these instructions
+    /// but could contain vector types.
+    fn allowed_instructions(&self) -> InstructionKinds {
+        InstructionKinds::all()
+    }
 }
 
 /// The default configuration.
@@ -323,6 +361,7 @@ pub struct SwarmConfig {
     // These fields are configured via `Arbitrary`
     pub max_types: usize,
     pub max_imports: usize,
+    pub max_tags: usize,
     pub max_funcs: usize,
     pub max_globals: usize,
     pub max_exports: usize,
@@ -345,6 +384,7 @@ pub struct SwarmConfig {
     pub memory64_enabled: bool,
     pub min_types: usize,
     pub min_imports: usize,
+    pub min_tags: usize,
     pub min_funcs: usize,
     pub min_globals: usize,
     pub min_exports: usize,
@@ -358,6 +398,8 @@ pub struct SwarmConfig {
     pub memory_offset_choices: (u32, u32, u32),
     pub memory_max_size_required: bool,
     pub simd_enabled: bool,
+    pub relaxed_simd_enabled: bool,
+    pub exceptions_enabled: bool,
     pub allow_start_export: bool,
     pub max_type_size: u32,
     pub canonicalize_nans: bool,
@@ -373,6 +415,7 @@ impl<'a> Arbitrary<'a> for SwarmConfig {
         Ok(SwarmConfig {
             max_types: u.int_in_range(0..=MAX_MAXIMUM)?,
             max_imports: u.int_in_range(0..=MAX_MAXIMUM)?,
+            max_tags: u.int_in_range(0..=MAX_MAXIMUM)?,
             max_funcs: u.int_in_range(0..=MAX_MAXIMUM)?,
             max_globals: u.int_in_range(0..=MAX_MAXIMUM)?,
             max_exports: u.int_in_range(0..=MAX_MAXIMUM)?,
@@ -384,8 +427,9 @@ impl<'a> Arbitrary<'a> for SwarmConfig {
             max_tables,
             max_memory_pages: u.arbitrary()?,
             min_uleb_size: u.int_in_range(0..=5)?,
-            bulk_memory_enabled: u.arbitrary()?,
+            bulk_memory_enabled: reference_types_enabled || u.arbitrary()?,
             reference_types_enabled,
+            simd_enabled: u.arbitrary()?,
             max_aliases: u.int_in_range(0..=MAX_MAXIMUM)?,
             max_nesting_depth: u.int_in_range(0..=10)?,
 
@@ -394,6 +438,7 @@ impl<'a> Arbitrary<'a> for SwarmConfig {
             // implemented yet so they're turned off by default.
             min_types: 0,
             min_imports: 0,
+            min_tags: 0,
             min_funcs: 0,
             min_globals: 0,
             min_exports: 0,
@@ -407,7 +452,8 @@ impl<'a> Arbitrary<'a> for SwarmConfig {
             max_modules: 0,
             memory_offset_choices: (75, 24, 1),
             allow_start_export: true,
-            simd_enabled: false,
+            relaxed_simd_enabled: false,
+            exceptions_enabled: false,
             memory64_enabled: false,
             max_type_size: 1000,
             module_linking_enabled: false,
@@ -545,6 +591,14 @@ impl Config for SwarmConfig {
         self.simd_enabled
     }
 
+    fn relaxed_simd_enabled(&self) -> bool {
+        self.relaxed_simd_enabled
+    }
+
+    fn exceptions_enabled(&self) -> bool {
+        self.exceptions_enabled
+    }
+
     fn allow_start_export(&self) -> bool {
         self.allow_start_export
     }
